/**
  ******************************************************************************
  * @file    AzureClientApplication.h
  * @author  SRA - Central Labs
  * @version V5.0.0
  * @date    18-March-2020
  * @brief   Header file for Main MQTT Application
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under Ultimate Liberty license
  * SLA0055, the "License"; You may not use this file except in compliance with
  * the License. You may obtain a copy of the License at:
  *                             www.st.com/SLA0055
  *
  ******************************************************************************
 */
#ifndef _AZURE_CLIENT_APPLICATION_H
#define _AZURE_CLIENT_APPLICATION_H

#ifdef __cplusplus
extern "C" {
#endif
#include "cJSON.h"
#include "azure_c_shared_utility/crt_abstractions.h"
#include "iothub_client_version.h"


/* enum values are in lower case per design */
#define FIRMWARE_UPDATE_STATUS_VALUES \
        Running, \
        Paused, \
        Rebooting, \
        Downloading, \
        DownloadFailed, \
        DownloadComplete, \
        Applying, \
        ApplyFailed, \
        ApplyComplete, \
        Ended
MU_DEFINE_ENUM(FIRMWARE_UPDATE_STATUS, FIRMWARE_UPDATE_STATUS_VALUES)
typedef void (*IOTConnectCallback)(char *PayLoad);
extern void TwinUpdateCallback(char *payload);
extern char* calling_type(int XX);
extern void SYNC_Resp(char *replay);
extern void DeviceCallback(char *payload);
extern void device_Resp(char *replay);
extern int SendToAzure(char *input);
extern void AzureClientApplication(void);
extern void SendSensorData(char *input);
extern void Received_cmd(char *payload);
extern void SendAck(char *Ack_Data, char *Ack_time, int messageType);
extern void Init(char *Sid, char *ConnectionString, char *sdk, IOTConnectCallback DeviceCallback,
		IOTConnectCallback TwinUpdateCallback);
extern void ReportState(FIRMWARE_UPDATE_STATUS status);
extern void WaitAllTheMessages(void);
extern void Dispose();
extern void GetAllTwins();
extern void differenceBetweenTimePeriod(RTC_TimeTypeDef start,
		RTC_TimeTypeDef stop, RTC_TimeTypeDef *diff);
#ifdef __cplusplus
}
#endif

#endif
