/**
 ******************************************************************************
  * @file    STM32CubeInterface.h
  * @author  SRA - Central Labs
  * @version V5.0.0
  * @date    18-March-2020
  * @brief   Header file for STM32 Cube interface file
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under Ultimate Liberty license
  * SLA0055, the "License"; You may not use this file except in compliance with
  * the License. You may obtain a copy of the License at:
  *                             www.st.com/SLA0055
  *
  ******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __STM32CUBEINTERFACE_H
#define __STM32CUBEINTERFACE_H

/* Includes ------------------------------------------------------------------*/
#include <time.h>
#include "TargetFeatures.h"
#include "azure_c_shared_utility/macro_utils.h"

/* Exported Types ----------------------------------------------------------- */
typedef enum
{
  None           = 0, 
  WEP            = 1,
  WPA_Personal   = 2,
  WPA2_Personal  = 3,
} WiFi_Priv_Mode;

typedef struct {
  char ssid[40];
  char seckey[40];
  WiFi_Priv_Mode mode;
} WIFI_CredAcc_t;

#define AZURE1_OTA_STATUS \
  OTA_STATUS_NULL, \
  OTA_STATUS_START, \
  OTA_STATUS_RUNNIG, \
  OTA_STATUS_END, \
  OTA_STATUS_ERROR
MU_DEFINE_ENUM(AZURE1_OTA, AZURE1_OTA_STATUS)

#define AZURE1_ERROR_VALUES \
  AZURE_ERR_NULL, \
  AZURE_ERR_PLATFORM_INIT, \
  AZURE_ERR_FOTA, \
  AZURE_ERR_BL_COMPLIANCE, \
  AZURE_ERR_IOT_START, \
  AZURE_ERR_NFC, \
  AZURE_ERR_WIFI_CRED, \
  AZURE_ERR_WIFI, \
  AZURE_ERR_WIFI_CON, \
  AZURE_ERR_MALLOC, \
  AZURE_ERR_REGISTRATION, \
  AZURE_ERR_NTP
MU_DEFINE_ENUM(AZURE1_ERROR, AZURE1_ERROR_VALUES)

/* Exported variables --------------------------------------------------------*/
extern volatile uint32_t ButtonPressed;
extern volatile uint32_t SendData;
extern uint8_t macAddress[6];

/* Exported functions ------------------------------------------------------- */
extern void RTC_CalendarShow(uint8_t *showtime, uint8_t *showdate);
extern time_t TimingSystemGetSystemTime(void);
extern int TimingSystemSetSystemTime(time_t epochTimeNow);
extern time_t SynchronizationAgentConvertNTPTime2EpochTime(uint8_t* pBufferTimingAnswer,size_t sizeBuffer);
extern void StartTimer1(void);
extern void StopTimer1(void);
extern void AzureExit(AZURE1_ERROR Value);
extern AZURE1_OTA FOTACallback(char * hostname,char type, uint32_t  port_num,char * path);

#endif /* __STM32CUBEINTERFACE_H */
/******************* (C) COPYRIGHT 2018 STMicroelectronics *****END OF FILE****/
