/**
  ******************************************************************************
  * @file    -AzureClient_mqtt_DM_TM.c
  * @author  -CSS - softweb solutions
  * @modify  -18-Feb-2021
  * @brief   -Source of STM32L4 IoTConnect SDK
  ******************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "STM32CubeInterface.h"

#include "azure_prov_client/prov_device_ll_client.h"
#include "azure_prov_client/prov_security_factory.h"
#include "azure_prov_client/prov_transport_mqtt_client.h"
#include "azure_prov_client/prov_transport_mqtt_ws_client.h"
#include "iothub_message.h"
#include "iothub_client_options.h"
#include "azure_c_shared_utility/crt_abstractions.h"
#include "azure_c_shared_utility/platform.h"
#include "iothubtransportmqtt.h"
#include "AzureClientApplication.h"
extern char certificates[];

char CRT_PEM[];
char KEY_PEM[];
#include "serializer.h"
#include "serializer_devicetwin.h"
#include "jsondecoder.h"


#ifdef AZURE_PRODUCE_LEAF_CERTIFICATES
  #include "hsm_client_riot.h"
  extern char* dps_hsm_riot_create_leaf_cert_signer(HSM_CLIENT_HANDLE handle, const char* common_name);
#endif /* AZURE_PRODUCE_LEAF_CERTIFICATES */

#if !defined(MBEDTLS_CONFIG_FILE)
  #include "mbedtls/config.h"
#else
  #include MBEDTLS_CONFIG_FILE
#endif

#include "mbedtls/entropy_poll.h"

// ####### IOTCONNECT CONFIGURATION #########
char *IC_SID=NULL, *IC_UNIQUEID=NULL;
int IsDebug = 0;
int USE_Cert = 0;
bool Flag_99 = true;
bool Flag_DF = true;
unsigned long int DF_last = 0;
char *DTG = NULL;
// time format : YYYY-MM-DDTHH:MM:SS.FFFZ eg. 2020-01-21T22:57:59.968Z
static const char TIME_FORMAT[]="%04u-%02u-%02uT%02u:%02u:%02u.%03uZ";
RTC_TimeTypeDef diff, oldTime,  newTime;
RTC_DateTypeDef lastDate, newDate;

typedef struct Sync_Resp{
	int ec;               //root..info
    int ct;
    int at;
    int dvc;
    int df;
    char *dvc_ID[10] ;
    char *tag[10] ;

};
struct Sync_Resp SYNC_resp;

typedef struct CLIENT_SAMPLE_INFO_TAG
{
  unsigned int SleepTime;
  char* IoTHubUri;
  char* DeviceId;
} CLIENT_SAMPLE_INFO;

/**********  below parameters useful for hardware use only **********************************/
BEGIN_NAMESPACE(AzureModel);

DECLARE_MODEL(Azure1_t,
/* Event data: Temperature, Humidity... */
WITH_DATA(ascii_char_ptr, deviceId),
WITH_DATA(int, messageId),
WITH_DATA(float, Temperature),
WITH_DATA(float, Humidity),
WITH_DATA(float, Pressure),
WITH_DATA(int, accX),
WITH_DATA(int, accY),
WITH_DATA(int, accZ),
WITH_DATA(int, gyrX),
WITH_DATA(int, gyrY),
WITH_DATA(int, gyrZ),
WITH_DATA(int, magX),
WITH_DATA(int, magY),
WITH_DATA(int, magZ),
WITH_DATA(EDM_DATE_TIME_OFFSET, ts),
/* Methods */
WITH_METHOD(Reboot),
WITH_METHOD(Quit),
WITH_METHOD(PausePlay),
WITH_METHOD(LedOnOff),
WITH_METHOD(LedBlink),
/* Reported Properties */
/* Register Direct Methods with IoT Hub */
WITH_REPORTED_PROPERTY(ascii_char_ptr_no_quotes, SupportedMethods),
/* Telemetry Interval in Seconds... value from 1 to: 0xFFFF/2000 = About 30 Seconds */
WITH_REPORTED_PROPERTY(int,TelemetryInterval),
WITH_REPORTED_PROPERTY(ascii_char_ptr,AzureStatus),
WITH_REPORTED_PROPERTY(ascii_char_ptr, AzureFwVersion)
);

/********************************************************************************************/

END_NAMESPACE(AzureModel);

Azure1_t *Azure1;

typedef struct EVENT_INSTANCE_TAG
{
  IOTHUB_MESSAGE_HANDLE messageHandle;
  size_t messageTrackingId;  // For tracking the messages within the user callback.
  void *this;  // For Memory management (free)
} EVENT_INSTANCE;

IOTHUB_CLIENT_LL_HANDLE iotHubClientHandle;

static bool g_continueRunning;
static bool g_reboot     = false;
static bool g_ExecuteOTA =false;
/* Counters for Sent/Received Status */
static size_t StatusBackCount = 0;
static size_t SentStatusCount = 0;
/* Counters for Sent/Received Messages */
static size_t MessagesBackCount = 0;
static size_t SentMessagesCount = 0;

static EDM_DATE_TIME_OFFSET GetDateTimeOffset(time_t time)
{
  struct tm *newTime;
  EDM_DATE_TIME_OFFSET dateTimeOffset;
  newTime = gmtime(&time);
  dateTimeOffset.dateTime = *newTime;
  dateTimeOffset.fractionalSecond = 0;
  dateTimeOffset.hasFractionalSecond = 0;
  dateTimeOffset.hasTimeZone = 0;
  dateTimeOffset.timeZoneHour = 0;
  dateTimeOffset.timeZoneMinute = 0;
  return dateTimeOffset;
}

/*! Object  - will Get RTC time
 *  \param  - need a pointer buffer with size
 *  \return - None
 */
static void FormatTime(char* buf, size_t bufSize) {
	  RTC_DateTypeDef sdatestructureget;
	  RTC_TimeTypeDef stimestructureget;

	  /* Get the RTC current Time */
	  HAL_RTC_GetTime(&TargetBoardFeatures.RtcHandle, &stimestructureget, RTC_FORMAT_BIN);
	  /* Get the RTC current Date */
	  HAL_RTC_GetDate(&TargetBoardFeatures.RtcHandle, &sdatestructureget, RTC_FORMAT_BIN);
	  sprintf_s(buf, bufSize, TIME_FORMAT,
	  		  2000 + sdatestructureget.Year,
			  	  	 sdatestructureget.Month,
					 sdatestructureget.Date,
					 stimestructureget.Hours,
					 stimestructureget.Minutes,
					 stimestructureget.Seconds,
			  1000 * stimestructureget.SubSeconds / stimestructureget.SecondFraction
	  		  );
}

/*! Object  - will Dispose the connection with IoTConnect cloud
 *  \param  - None
 *  \return - None
 */
void Dispose(void){
	IoTHubClient_LL_Destroy(iotHubClientHandle);
    Flag_99 = false;
    cJSON *obj;
    obj = cJSON_CreateObject();
    cJSON_AddStringToObject(obj, "sid",IC_SID);
    cJSON_AddStringToObject(obj, "uniqueId",IC_UNIQUEID);
    cJSON_AddStringToObject(obj, "guid","");
    cJSON_AddBoolToObject(obj, "ack",false);
    cJSON_AddStringToObject(obj, "ackId","");
    cJSON_AddBoolToObject(obj, "command", false);
    cJSON_AddStringToObject(obj, "cmdType","0x16");
    DeviceCallback(cJSON_Print(obj));
    HAL_Delay(2000);
    cJSON_Delete(obj);
    AZURE_PRINTF("\r\n\t INFO_DC01 [ %s - %s ] :Device already disconnected:\r\n", IC_SID, IC_UNIQUEID);
}

/**
 * @brief CallBack received for Device Twin status update
 * @param int status_code Status received
 * @param void* userContextCallback unused
 * @retval None
 */
static void DeviceTwinCallback(int status_code, __attribute__((unused)) void* userContextCallback){
	
	StatusBackCount++;
	AZURE_PRINTF("-->DeviceTwin CallBack [%d]: Status_code = %d\r\n", StatusBackCount, status_code);
}

/**
 * @brief CallBack received at the beginning on when there is a change on desired properties
 * @param DEVICE_TWIN_UPDATE_STATE status_code Status received
 * @param unsigned char* payload Payload received (json)
 * @param size_t size size of the Payload received
 * @param void* userContextCallback pointer to the model instance
 * @retval None
 */
static void DeviceTwinCallbackStatus(DEVICE_TWIN_UPDATE_STATE status_code,  const unsigned char* payload, size_t size, void* userContextCallback){
	
	/* Only for Debug */
	//AZURE_PRINTF("DeviceTwinCallbackStatus Method payload: %.*s\r\nStatus_code = %d\r\n", size,(const char*)payload,status_code);

	/* query the DeviceTwin Status */
	{
		JSON_DECODER_RESULT JSONDec;
		MULTITREE_HANDLE multiTreeHandle;
		char* temp = malloc(size + 1);
		if (temp == NULL) {
			AZURE_PRINTF("Err: failed to malloc\r\n");
		return;
		}
    
		/* We need to add the missing termination char */
		(void)memcpy(temp, payload, size);
		temp[size] = '\0';
		if((JSONDec = JSONDecoder_JSON_To_MultiTree(temp,&multiTreeHandle))!=JSON_DECODER_OK) {

			AZURE_PRINTF("Err: Decoding JSON Code=%d\r\n",JSONDec);
			free(temp);
			return;
		}
		else {
			AZURE_PRINTF("JSON Decoded\r\n");
			TwinUpdateCallback(payload);
		}
		
		MultiTree_Destroy(multiTreeHandle);
		free(temp);
	}
}



/* @brief Callback received when a message is received from IoT
 * @param IOTHUB_CLIENT_CONFIRMATION_RESULT result confirmation result from IoTs
 * @param void* userContextCallback pointer to event instance
 * @retval None
 */
static void SendConfirmationCallback(IOTHUB_CLIENT_CONFIRMATION_RESULT result, void* userContextCallback){
	
	EVENT_INSTANCE* eventInstance = (EVENT_INSTANCE*)userContextCallback;

	AZURE_PRINTF("Confirmation received for message [%d] with result = %s\r\n", eventInstance->messageTrackingId, MU_ENUM_TO_STRING(IOTHUB_CLIENT_CONFIRMATION_RESULT, result));
	MessagesBackCount++;
	free(eventInstance->this);
}


/* @param IOTHUB_MESSAGE_HANDLE message Pointer to the received message
 * @param void* userContextCallback pointer to the model instance
 * @retval IOTHUBMESSAGE_DISPOSITION_RESULT Results of the messagge (Abandoned/Accepted/...)
 */
static IOTHUBMESSAGE_DISPOSITION_RESULT ReceiveMessageCallback(IOTHUB_MESSAGE_HANDLE message, void* userContextCallback){
	
    IOTHUBMESSAGE_DISPOSITION_RESULT result;
    const unsigned char* buffer;
    size_t size;
    int in_CT=0;
    if (IoTHubMessage_GetByteArray(message, &buffer, &size) != IOTHUB_MESSAGE_OK){
        AZURE_PRINTF("unable to retrieve the message data\r\n");
        result = IOTHUBMESSAGE_ABANDONED;
    }
	else {
		/*buffer is not zero terminated*/
    	char* temp = malloc(size + 1);
    	if (temp == NULL) {
			AZURE_PRINTF("Err: failed to malloc\r\n");
        	result = IOTHUBMESSAGE_ABANDONED;
    	}
		else {
			EXECUTE_COMMAND_RESULT executeCommandResult;
			AZURE_PRINTF("\r\n*** Received Message with Data: <<<%.*s>>> & Size=%d\r\n\n", (int)size, buffer, (int)size);

			cJSON *root = NULL;
    		root = cJSON_Parse(buffer);
  		    if(cJSON_GetObjectItem(root, "d")){
   		    	cJSON *in_CMD = cJSON_Parse(buffer);
   		    	char *in_data =  cJSON_GetObjectItem(in_CMD, "d");
   		    	if(cJSON_IsObject(in_data)){
   		    		cJSON *value = cJSON_GetObjectItem(in_data, "ct");
   		    		in_CT = value->valueint;

  		    		if(in_CT == 200)
   		    			SYNC_Resp(buffer);
   		    		else if(in_CT == 204)
   		    			device_Resp(buffer);
   		    		else if(in_CT == 201){
   		    			char *in_data;
   		    			cJSON *in_CMD, *meta, *value;
   		    			in_CMD = cJSON_Parse(buffer);
   		    			in_data =  cJSON_GetObjectItem(in_CMD, "d");
   		    			if(cJSON_IsObject(in_data)){
   		    				meta = cJSON_GetObjectItem(in_data, "meta");
   		    				if(cJSON_IsObject(meta)){
   		    					value = cJSON_GetObjectItem(meta, "df");
   		    					SYNC_resp.df = value->valueint;
   		    				}
   		    			}
   		    		}
   		    	}
   		    }
   		    else if(cJSON_GetObjectItem(root, "cmdType")){
   		    	cJSON *Cdata;
   		        char *cmdValue;
   		        cmdValue = (cJSON_GetObjectItem(root, "cmdType"))->valuestring;
   		        if( (strcmp(cmdValue,"0x02") == 0) || (strcmp(cmdValue,"0x01")==0) ){
   		        	Cdata =  cJSON_GetObjectItem(root, "data");
   		    		if(cJSON_IsObject(Cdata))
   		    			DeviceCallback(cJSON_Print(Cdata));
   		        }
   		    	else{
   		    		Received_cmd(buffer);
    		    	}
    		}
    		else {
    		   	AZURE_PRINTF("\r\n\t ::  Received UNKNOWN MSG :: \r\n");
    		}
    		
			(void)memcpy(temp, buffer, size);
   		    temp[size] = '\0';
   		    executeCommandResult = EXECUTE_COMMAND(userContextCallback, temp);
   		    result =
 		    		(executeCommandResult == EXECUTE_COMMAND_ERROR) ? IOTHUBMESSAGE_ABANDONED :
    				(executeCommandResult == EXECUTE_COMMAND_SUCCESS) ? IOTHUBMESSAGE_ACCEPTED :
						IOTHUBMESSAGE_REJECTED;
   		    free(temp);
   		    buffer = NULL;
		}
    }
	return result;
}


/*! Object  - will get the SDK command from C2D
 *  \param  - Payload of command
 *  \return - None
 */
void Received_cmd(char *payload){

	char *new_data;
	cJSON *value,*in_obj;
  	in_obj = cJSON_Parse(payload);
	value = cJSON_GetObjectItem(in_obj, "cmdType");
	new_data = value->valuestring;
	if (strcmp(new_data,"0x10") == 0){
		char *output =  calling_type(201);
		if(SendToAzure(output)){
			AZURE_PRINTF("\r\n\t INFO_ [ %s - %s ] :  Publish Attribute_info:\r\n", IC_SID, IC_UNIQUEID);
		}else{
			AZURE_PRINTF("\r\n\t ERR_ [ %s - %s ] :  Publish Attribute_info failed :\r\n", IC_SID, IC_UNIQUEID);
		}
		if(IsDebug){
			AZURE_PRINTF("\r\n\tINFO_CM03 0x10 [ %s - %s ] :  ATTRIBUTE_INFO_UPDATE\r\n", IC_SID, IC_UNIQUEID);
			}
	    }
	else if(strcmp(new_data,"0x11") == 0){
		if(IsDebug){
			AZURE_PRINTF("\r\n\tINFO_CM04 0x11 [ %s - %s ] :  SETTING_INFO_UPDATE\r\n", IC_SID, IC_UNIQUEID);
	        }
	    }
	else if(strcmp(new_data,"0x12") == 0) {
		if(IsDebug){
			AZURE_PRINTF("\r\n\tINFO_CM05 0x12 [ %s - %s ] :  PASSWORD_INFO_UPDATE\r\n", IC_SID, IC_UNIQUEID);
	        }
	    }
	else if(strcmp(new_data,"0x13") == 0) {
	    if(IsDebug){
	        AZURE_PRINTF("\r\n\tINFO_CM06 0x13 [ %s - %s ] :  DEVICE_INFO_UPDATE\r\n", IC_SID, IC_UNIQUEID);
	        }
	    }
	else if(strcmp(new_data,"0x15") == 0) {
	    if(IsDebug){
	        AZURE_PRINTF("\r\n\tINFO_CM07 0x15 [ %s - %s ] :  RULE_INFO_UPDATE\r\n", IC_SID, IC_UNIQUEID);
	    	}
	    }
	else if(strcmp(new_data,"0x99") == 0) {
	    cJSON *obj;
	    obj = cJSON_CreateObject();
	    cJSON_AddStringToObject(obj, "sid",IC_SID);
	    cJSON_AddStringToObject(obj, "uniqueId",IC_UNIQUEID);
	    cJSON_AddStringToObject(obj, "guid","");
	    cJSON_AddBoolToObject(obj, "ack",false);
	    cJSON_AddStringToObject(obj, "ackId","");
	    cJSON_AddBoolToObject(obj, "command", false);
	    cJSON_AddStringToObject(obj, "cmdType","0x16");
	    DeviceCallback(cJSON_Print(obj));
	    HAL_Delay(2000);
	    cJSON_Delete(obj);
	  	IoTHubClient_LL_Destroy(iotHubClientHandle);
	    Flag_99 = false;
	
		if(IsDebug){
	    	AZURE_PRINTF("\r\n\tINFO_CM08 0x99 [ %s - %s ] :  STOP_SDK_CONNECTION\r\n", IC_SID, IC_UNIQUEID);
	    }
	}
}


/*! Object  - will send the command ACK from D2C
 *  \param  - @Ack_Data - data from firm with ACK_ID
 *  		- @msgType - ACK for OTA or device Command
 *  \return - None
 */
void SendAck(char *Ack_Data, char *Ack_time, int msgType){
	
	cJSON *ACK, *Ack_json;
	ACK = cJSON_CreateObject();
	Ack_json = cJSON_Parse(Ack_Data);
	if(! cJSON_IsObject(Ack_json)){
		AZURE_PRINTF("\r\n\t ERR_CM03 [ %s - %s ] : Invalid data type to send the acknowledgment. It should be in 'object' type\r\n", IC_SID, IC_UNIQUEID);	
		return ;
	}
	else{
		cJSON_AddStringToObject(ACK, "uniqueId", IC_UNIQUEID);
		cJSON_AddNumberToObject(ACK, "mt", msgType);
		cJSON_AddStringToObject(ACK, "sid", IC_SID);
		cJSON_AddStringToObject(ACK, "dt", Ack_time);
		cJSON_AddItemToObject(ACK, "d", Ack_json);

		char *Ack_Json_Data = cJSON_Print(ACK);	
		HAL_Delay(10);

		if(SendToAzure(Ack_Json_Data)){
			AZURE_PRINTF("IoTHubClient_LL_SendEventAsync accepted message [%d] for transmission to IoT Hub.\r\n", SentMessagesCount);
			AZURE_PRINTF("\r\n\t INFO_CM10 [ %s - %s ] :Command acknowledgement success :\r\n", IC_SID, IC_UNIQUEID);
		}
		else{
			AZURE_PRINTF("Err: IoTHubClient_LL_SendEventAsync..........FAILED!\r\n");
		}

		cJSON_Delete(ACK);
		return 0;
	}
}


/**
  * @brief  this will init core platform of controller family
  * @param  None
  * @retval None
  */
void AzureClientApplication(void){

	if (platform_init() != 0){
		AZURE_PRINTF("Failed to initialize the platform.\r\n");
		AzureExit(AZURE_ERR_PLATFORM_INIT);
	}
	else {
		char *RegistrationName = TargetBoardFeatures.RegistrationName;
		CLIENT_SAMPLE_INFO user_ctx;
		g_continueRunning = true;
		AZURE_PRINTF("\n\tPlatform Init Done\r\n");
	}
}


/**
 * @brief  this will init application for IoTConnect
 * @param
 	  Input   : sid, ConnectionString, sdkOptions as explained main file
 * @retval None
 */
void Init(char *sid, char *ConnectionString, char *sdk, IOTConnectCallback CallbackMessage, IOTConnectCallback TwinCallbackMessage ) {
	
	if(Flag_99){
		if((strcmp(sid,"") == 0 ) || (strcmp(ConnectionString,"") == 0 )){
			if(strlen(sid)< 2)
				AZURE_PRINTF("\r\n\tERR_IN04 [ %s - %s ] : SID can't be blank\r\n", sid, IC_UNIQUEID);
			if(strlen(ConnectionString)< 2)
				AZURE_PRINTF("\r\n\tERR_IN05 [ %s - %s ] : ConnectionString can't be blank\r\n", sid, IC_UNIQUEID);
			return ;
		}

		IC_SID = sid;
		char CST[2000] = "";
		strcpy(CST, strstr(ConnectionString,"DeviceId"));
		IC_UNIQUEID = strtok(CST,"-"); IC_UNIQUEID = strtok(NULL,";");
		cJSON *value,*in_obj;
		in_obj = cJSON_Parse(sdk);
		value = cJSON_GetObjectItem(in_obj, "IsDebug");
		if(cJSON_IsNumber(value))
			IsDebug = value->valueint;


		if(USE_Cert){			/* Load certificates */
			if (IoTHubClient_LL_SetOption(iotHubClientHandle, OPTION_TRUSTED_CERT, certificates) != IOTHUB_CLIENT_OK ) {
				AZURE_PRINTF("Err: IoTHubClient_LL_SetOption(certificates)..........FAILED!\r\n");
				AzureExit(AZURE_ERR_IOT_START);
			}
			else {
				AZURE_PRINTF("IoTHubClient_LL_SetOption(certificates)...successful.\r\n");
			}
			
			if( (IoTHubClient_LL_SetOption(iotHubClientHandle, OPTION_X509_CERT, CRT_PEM) != IOTHUB_CLIENT_OK) ||
					(IoTHubClient_LL_SetOption(iotHubClientHandle, OPTION_X509_PRIVATE_KEY, KEY_PEM) != IOTHUB_CLIENT_OK)){
					printf("failure to set options for x509, aborting\r\n");
					return;
			} 
			else{
				AZURE_PRINTF("IoTHubClient_LL_SetOption(device certificates)...successful.\r\n");
				if(IsDebug){
					AZURE_PRINTF("\r\n\tINFO_IN04 [ %s - %s ] : Initializing… \r\n", IC_SID, IC_UNIQUEID);
					AZURE_PRINTF("\r\n\tINFO_IN05 [ %s - %s ] : Connecting… \r\n", IC_SID, IC_UNIQUEID);
				}
			}
		}
		else{
			if((iotHubClientHandle = IoTHubClient_LL_CreateFromConnectionString(ConnectionString, MQTT_Protocol)) == NULL){
				AZURE_PRINTF("Err: iotHubClientHandle is NULL!\r\n");
				AZURE_PRINTF("\tchange AZUREDEVICECONNECTIONSTRING input from main.c file\r\n");
				AzureExit(AZURE_ERR_IOT_START);
			}
			else {
				AZURE_PRINTF("iotHubClientHandle Created\r\n");
				if(IsDebug){
					AZURE_PRINTF("\r\n\tINFO_IN04 [ %s - %s ] : Initializing… \r\n", IC_SID, IC_UNIQUEID);
					AZURE_PRINTF("\r\n\tINFO_IN05 [ %s - %s ] : Connecting… \r\n", IC_SID, IC_UNIQUEID);
				}
			}
		}
	
	{
		int32_t KeepAlive = 2*60; /* 10 Minutes */
		if(IoTHubClient_LL_SetOption(iotHubClientHandle, "keepalive", &KeepAlive)==IOTHUB_CLIENT_OK) {
			AZURE_PRINTF("iotHubClientSetOption keepalive Ok\r\n");
		}
	}

	/* Setting Message call back, so we can receive Commands. */
    if (IoTHubClient_LL_SetMessageCallback(iotHubClientHandle, ReceiveMessageCallback, Azure1) != IOTHUB_CLIENT_OK){
		AZURE_PRINTF("Err: IoTHubClient_LL_SetMessageCallback..........FAILED!\r\n");
		AzureExit(AZURE_ERR_IOT_START);
    }
	else {
		AZURE_PRINTF("IoTHubClient_LL_SetMessageCallback...successful.\r\n");
		cJSON *obj;
		obj = cJSON_CreateObject();
		cJSON_AddStringToObject(obj, "sid",IC_SID);
		cJSON_AddStringToObject(obj, "uniqueId",IC_UNIQUEID);
		cJSON_AddStringToObject(obj, "guid","");
		cJSON_AddBoolToObject(obj, "ack",false);
		cJSON_AddStringToObject(obj, "ackId","");
		cJSON_AddBoolToObject(obj, "command", true);
		cJSON_AddStringToObject(obj, "cmdType","0x16");
		DeviceCallback(cJSON_Print(obj));
		HAL_Delay(2000);
		cJSON_Delete(obj);
    }

    if(IoTHubClient_LL_SetDeviceTwinCallback(iotHubClientHandle, DeviceTwinCallbackStatus, Azure1) != IOTHUB_CLIENT_OK){
		AZURE_PRINTF("Err: IoTHubClient_LL_SetDeviceTwinCallback..........FAILED!\r\n");
		AzureExit(AZURE_ERR_IOT_START);
    }
	else {
		AZURE_PRINTF("IoTHubClient_LL_SetDeviceTwinCallback...successful.\r\n");
    }

	char *output =  calling_type(200);
    if(SendToAzure(output)){
		if(IsDebug)
			AZURE_PRINTF("\r\n\t INFO_ [ %s - %s ] :  Publish Hello_json_data:\r\n", IC_SID, IC_UNIQUEID);
    }
	else{
		if(IsDebug)
    	  AZURE_PRINTF("\r\n\t ERR_ [ %s - %s ] :  Publish Hello_json_data failed :\r\n", IC_SID, IC_UNIQUEID);
    }
	
	unsigned int xyz =0;
	while(xyz <120){
		xyz++;
		IoTHubClient_LL_DoWork(iotHubClientHandle);
		HAL_Delay(100);
		}
    
	if(SYNC_resp.dvc == 1){
		output =  calling_type(204);
		if(SendToAzure(output)){
			if(IsDebug)
				AZURE_PRINTF("\r\n\t INFO_ [ %s - %s ] :  Publish device_json_data:\r\n", IC_SID, IC_UNIQUEID);

		}else{
			if(IsDebug)
				AZURE_PRINTF("\r\n\t ERR_ [ %s - %s ] :  Publish device_json_data failed :\r\n", IC_SID, IC_UNIQUEID);
		}
    }
    xyz=0;
	while(xyz <120){
		xyz++;
		IoTHubClient_LL_DoWork(iotHubClientHandle);
		HAL_Delay(100);
		}
	}
}


/*! Object  - will get all device twin from C2D
 *  \param  - None
 *  \return - None
 */
void GetAllTwins(void){
   
   if(IoTHubClient_LL_SetDeviceTwinCallback(iotHubClientHandle, DeviceTwinCallbackStatus, Azure1) != IOTHUB_CLIENT_OK){
		AZURE_PRINTF("Err: IoTHubClient_LL_SetDeviceTwinCallback..........FAILED!\r\n");
		AzureExit(AZURE_ERR_IOT_START);
    }
	else {
		AZURE_PRINTF("IoTHubClient_LL_SetDeviceTwinCallback...successful.\r\n");
    }
}


/*! Object  - will save all SYNC response in local memory
 *  \param  - response from C2D
 *  \return - None
 */
void SYNC_Resp(char *replay) {

	char *in_data;
 	cJSON *in_CMD, *has, *meta, *value;

	in_CMD = cJSON_Parse(replay);
	in_data =  cJSON_GetObjectItem(in_CMD, "d");
	if(cJSON_IsObject(in_data)){
	    value = cJSON_GetObjectItem(in_data, "ct");
	    SYNC_resp.ct = value->valueint;
	    value = cJSON_GetObjectItem(in_data, "ec");
	    SYNC_resp.ec = value->valueint;
	    meta = cJSON_GetObjectItem(in_data, "meta");
	    if(cJSON_IsObject(meta)){
	    	value = cJSON_GetObjectItem(meta, "dtg");
	    	DTG = value->valuestring;
	    	value = cJSON_GetObjectItem(meta, "df");
	    	SYNC_resp.df = value->valueint;
			value = cJSON_GetObjectItem(meta, "tg");
			SYNC_resp.tag[0] = value->valuestring;
	    }
	    has = cJSON_GetObjectItem(in_data, "has");
	    if(cJSON_IsObject(has)){
		    value = cJSON_GetObjectItem(has, "d");
		    SYNC_resp.dvc = value->valueint;
	    }
	}
	else 
		return;  
	if(IsDebug){
		switch(SYNC_resp.ec){
			case 0: {
				AZURE_PRINTF("\r\n\tINFO_IN08 [ %s - %s ] :  Response Code : 0 'OK'\r\n", IC_SID, IC_UNIQUEID);
				break;
			}
			case 1: {
				AZURE_PRINTF("\r\n\tINFO_IN09 [ %s - %s ] :  Response Code : 1 'DEVICE_NOT_REGISTERED'\r\n", IC_SID, IC_UNIQUEID);
				break;
			}
			case 2: {
				AZURE_PRINTF("\r\n\tINFO_IN10 [ %s - %s ] :  Response Code : 2 'AUTO_REGISTER'\r\n", IC_SID, IC_UNIQUEID);
				break;
			}
			case 3: {
				AZURE_PRINTF("\r\n\tINFO_IN11 [ %s - %s ] :  Response Code : 3 'DEVICE_NOT_FOUND'\r\n", IC_SID, IC_UNIQUEID);
				break;
			}
			case 4: {
				AZURE_PRINTF("\r\n\tINFO_IN12 [ %s - %s ] :  Response Code : 4 'DEVICE_INACTIVE'\r\n", IC_SID, IC_UNIQUEID);
				break;
			}
			case 5: {
				AZURE_PRINTF("\r\n\tINFO_IN13 [ %s - %s ] :  Response Code : 5 'OBJECT_MOVED'\r\n", IC_SID, IC_UNIQUEID);
				break;
			}
			case 6: {
			    AZURE_PRINTF("\r\n\tINFO_IN14 [ %s - %s ] :  Response Code : 6 'CPID_NOT_FOUND'\r\n", IC_SID, IC_UNIQUEID);
				break;
			}
			default: {
				AZURE_PRINTF("\r\n\tINFO_IN15 [ %s - %s ] :  Response Code : 'NO_RESPONSE_CODE_MATCHED'\r\n", IC_SID, IC_UNIQUEID);
				break;
			}
		}
	}
		//cJSON_Delete(in_CMD);
 }

/*! Object  - will publish all sensors data to IoTConnect cloud with help of SendToAzure()
 *  \param  - sensor data from firmware
 *  \return - None
 */
void SendSensorData(char *input){
	
	if(Flag_99){
		HAL_RTC_GetTime(&TargetBoardFeatures.RtcHandle, &newTime, RTC_FORMAT_BIN);
		differenceBetweenTimePeriod(newTime,oldTime,&diff);
		unsigned int TimeInSec = (diff.Hours *60*60) + (diff.Minutes *60) +(diff.Seconds);

		if(SYNC_resp.df < TimeInSec){
			char SD_time[24 + 1]; 	// time format : YYYY-MM-DDTHH:MM:SS.FFFZ eg. 2020-01-21T22:57:59.968Z
			FormatTime(SD_time, sizeof(SD_time));
			int x=0,y=0;
			char *in_time[10], *in_ID[10], *in_data[10];
			cJSON *value, *in_obj,*TO_HUB, *sdk, *device, *Arraydevice, *Arraydata;
			cJSON *in_att = cJSON_Parse(input);
			
			if (in_att == NULL)  {
				const char *error_ptr = cJSON_GetErrorPtr();
				if (error_ptr != NULL){
					AZURE_PRINTF(stderr, "Error before: %s\n", error_ptr);
				}
			}
			
			int Device = cJSON_GetArraySize(in_att);
			for(x=0; x<Device; x++){
				in_obj = cJSON_GetArrayItem(in_att,x);
				value = cJSON_GetObjectItem(in_obj, "uniqueId");
				if(cJSON_IsString(value))
					in_ID[x] = value->valuestring;
				else AZURE_PRINTF("\r\n\t ERR_SD08 [ %s - %s ] :  Missing required parameter 'uniqueId' \r\n", IC_SID, IC_UNIQUEID);
				
				value = cJSON_GetObjectItem(in_obj, "time");
				if(cJSON_IsString(value))
					in_time[x] = value->valuestring;
				else AZURE_PRINTF("\r\n\t ERR_SD07 [ %s - %s ] :  Missing required parameter 'time' \r\n", IC_SID, IC_UNIQUEID);
			
				in_data[x] =  cJSON_GetObjectItem(in_obj, "data");
				if(cJSON_IsObject(in_data[x]));
				else AZURE_PRINTF("\r\n\t ERR_SD06 [ %s - %s ] :  Missing required parameter 'data' \r\n", IC_SID, IC_UNIQUEID);
			}
	    
			in_time[0] = SD_time;
			TO_HUB = cJSON_CreateObject();
			cJSON_AddStringToObject(TO_HUB, "sid", IC_SID);
			cJSON_AddNumberToObject(TO_HUB, "mt", 0);
			cJSON_AddStringToObject(TO_HUB, "dt", in_time[0]);
			cJSON_AddStringToObject(TO_HUB, "dtg", DTG);
			cJSON_AddItemToObject(TO_HUB, "d", Arraydevice =cJSON_CreateArray());
			for(x=0; x<Device; x++){
				cJSON_AddItemToArray(Arraydevice,device = cJSON_CreateObject());
				cJSON_AddStringToObject(device, "id",in_ID[x]);
				for(y=0; y<10; y++){
					if(strcmp(in_ID[x], SYNC_resp.dvc_ID[y]) == 0)
						break;
					else
						continue;
				}
	    	
				if(SYNC_resp.dvc == 1)
					cJSON_AddStringToObject(device, "tg",SYNC_resp.tag[y]);
				else
					cJSON_AddStringToObject(device, "tg","");
					cJSON_AddStringToObject(device, "dt",in_time[0]);
					cJSON_AddItemToObject(device, "d",in_data[x]);
			}

			char *TO_HUB_data = cJSON_Print(TO_HUB);
			//AZURE_PRINTF("\r\n\t Published msg %s\r\n",TO_HUB_data);
			if (SendToAzure(TO_HUB_data)){
				AZURE_PRINTF("\r\n\t INFO_SD01 [ %s - %s ] :  Publish data:\r\n", IC_SID, IC_UNIQUEID);
				AZURE_PRINTF("IoTHubClient_LL_SendEventAsync accepted message [%d] for transmission to IoT Hub.\r\n", SentMessagesCount);
				Flag_DF = false;
			}else{
				AZURE_PRINTF("\r\n\t ERR_SD10 [ %s - %s ] :  Publish data failed :\r\n", IC_SID, IC_UNIQUEID);
			}

			lastDate = newDate;    oldTime = newTime;
			HAL_Delay(10);
			cJSON_Delete(TO_HUB);
		}
	}
	else 
		return;
}


/*! Object  - will publish data to IoTConnect cloud
 *  \param  - sensor data from SendSensorData()
 *  \return - 1/0 (as perPublished or failed)
 */
int SendToAzure(char *payload){

	EVENT_INSTANCE *messages;
	size_t destinationSize = strlen((const char*)payload);
	messages = (EVENT_INSTANCE *) calloc(1, destinationSize);
	if(messages==NULL) {
		AZURE_PRINTF("Err: Allocating Memory for messages to IoT Hub\r\n");
	    while(1) {;}
	}
	else {
		messages->this = (void *)messages;
    }
	
	if ((messages->messageHandle = IoTHubMessage_CreateFromByteArray(payload, destinationSize)) == NULL) {
		AZURE_PRINTF("Err: iotHubMessageHandle is NULL!\r\n");
	}
	else {
		if (IoTHubClient_LL_SendEventAsync(iotHubClientHandle, messages->messageHandle, SendConfirmationCallback, messages) != IOTHUB_CLIENT_OK) {
			IoTHubMessage_Destroy(messages->messageHandle);
			return 0;
		}
		else {
			AZURE_PRINTF("IoTHubClient_LL_SendEventAsync accepted message [%d] for transmission to IoT Hub.\r\n", SentMessagesCount);
			IoTHubMessage_Destroy(messages->messageHandle);
			return 1;
		}
	}
}


/*! Object  - will gave JSON for SYNC and device call with "mt" type
 *  \param  - MT { "allStatus" : 200, /"attribute" : 201, /"setting" : 202,
 *	 		       "rule" : 203, /"childDevice" : 204, /"all" : 210 }
 *  \return - data for publish
 */
char* calling_type(int MT){

	char tsStrBuf[24 + 1];
	FormatTime(tsStrBuf, sizeof(tsStrBuf));
	cJSON *Hello_json;
	Hello_json = cJSON_CreateObject();
    cJSON_AddStringToObject(Hello_json, "t", tsStrBuf);
    cJSON_AddStringToObject(Hello_json, "sid", IC_SID);
    cJSON_AddNumberToObject(Hello_json, "mt", MT);
    cJSON_AddNumberToObject(Hello_json, "v", 1);
    char *Hello_json_data = cJSON_Print(Hello_json);
    HAL_Delay(20);
    cJSON_Delete(Hello_json);
	return Hello_json_data;
}

/*! Object  - will gave Device SYNC response
 *  \param  - Payload from cloud response for device info
 *  \return - None
 */
void device_Resp(char *replay){
	
	char *in_data; int xx=0;
	cJSON *in_CMD, *has, *value, *obj ,*meta;
	in_CMD = cJSON_Parse(replay);
	in_data =  cJSON_GetObjectItem(in_CMD, "d");
	if(cJSON_IsObject(in_data)){
	    meta = cJSON_GetObjectItem(in_data, "meta");
	    if(cJSON_IsObject(meta)){
	    	value = cJSON_GetObjectItem(meta, "dtg");
	    	DTG = value->valuestring;}
		value = cJSON_GetObjectItem(in_data, "ct");
		SYNC_resp.ct = value->valueint;
		has = cJSON_GetObjectItem(in_data, "d");
		if(cJSON_IsArray(has)){
			for(xx=0; xx<cJSON_GetArraySize(has); xx++){
				obj = cJSON_GetArrayItem(has,xx);
				value = cJSON_GetObjectItem(obj, "id");
				SYNC_resp.dvc_ID[xx] = value->valuestring;
				value = cJSON_GetObjectItem(obj, "tg");
				SYNC_resp.tag[xx] = value->valuestring;
			}
		}
	}
	else 
		return;
}

/*! Object  - will gave difference between to time
 *  \param  - Payload from SYNC response for device
 *  \return - None (time in RTC_TimeTypeDef *diff)
 */
void differenceBetweenTimePeriod(RTC_TimeTypeDef start,
									RTC_TimeTypeDef stop, RTC_TimeTypeDef *diff) {
   
   while (stop.Seconds > start.Seconds) {
      --start.Minutes;
      start.Seconds += 60;
   }
   diff->Seconds = start.Seconds - stop.Seconds;
   while (stop.Minutes > start.Minutes) {
      --start.Hours;
      start.Minutes += 60;
   }
   diff->Minutes = start.Minutes - stop.Minutes;
   diff->Hours = (start.Hours - stop.Hours);
}


