/**
  ******************************************************************************
  * @file    tickcounter_STM32Cube.c
  * @author  SRA - Central Labs
  * @version V5.0.0
  * @date    18-March-2020
  * @brief   Tick Counter adapter 
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under Ultimate Liberty license
  * SLA0055, the "License"; You may not use this file except in compliance with
  * the License. You may obtain a copy of the License at:
  *                             www.st.com/SLA0055
  *
  ******************************************************************************
  */

#include <stdlib.h>
#include <stdint.h>
#include "stm32l4xx_hal.h"
#ifdef UNUSED /* Prevent the redefinition from HAL to MS shared utilities */
#undef UNUSED
#endif
#include "azure_c_shared_utility/gballoc.h"
#include "azure_c_shared_utility/tickcounter.h"
#include "azure_c_shared_utility/xlogging.h"

typedef struct TICK_COUNTER_INSTANCE_TAG
{
	int dummy : 1;
} TICK_COUNTER_INSTANCE;

/**
 * @brief Function for allocating the Tick Counter Structure
 * @param None
 * @retval TICK_COUNTER_HANDLE pointer to the allocated Tick Counter structure
 */
TICK_COUNTER_HANDLE tickcounter_create(void)
{
  TICK_COUNTER_INSTANCE* result = (TICK_COUNTER_INSTANCE*)malloc(sizeof(TICK_COUNTER_INSTANCE));
  if (result == NULL) {
    /* add any per instance initialization (starting a timer for example) here if needed (most platforms will not need this) */
    LogError("Cannot create tick counter");
  }
  return result;
}

/**
 * @brief Function for deallocating the Tick Counter Structure
 * @param TICK_COUNTER_HANDLE pointer to the allocated Tick Counter structure
 * @retval None
 */
void tickcounter_destroy(TICK_COUNTER_HANDLE tick_counter)
{
  if (tick_counter != NULL) {
    /* add any per instance de-initialization here (stopping the time) if needed (most platforms will not need this) */
    free(tick_counter);
  }
}

/**
 * @brief Function for geting the Tick counter values in milliseconds
 * @param TICK_COUNTER_HANDLE pointer to the allocated Tick Counter structure
 * @param tickcounter_ms_t* current_ms Tick counter value in milliseconds
 * @retval int OK/Error (0/!=0)
 */
int tickcounter_get_current_ms(TICK_COUNTER_HANDLE tick_counter, tickcounter_ms_t* current_ms)
{
  int result =0;
  if (tick_counter == NULL) {
    LogError("tickcounter failed: Invalid Arguments.");
    result = __LINE__;
  } else {
    /* call here any platform/OS specific function to get the millisecond tick and perform any conversions */
    /* Important: The HAL_GetTick return one uint32_t Values .. so after 49 days it will restart */
    *current_ms = HAL_GetTick();
  }
  return result;
}
/******************* (C) COPYRIGHT 2018 STMicroelectronics *****END OF FILE****/

